/**
 * Javascript for time tracking.
 */
(function ($, Drupal, drupalSettings) {
  var timer;
  var totalActiveTime;
  // Check for the drupal settings.
  if (drupalSettings.hasOwnProperty('openlucius_timetracker')) {

    // Check the node ip and port.
    if (drupalSettings.openlucius_timetracker.hasOwnProperty('node_ip') && drupalSettings.openlucius_timetracker.hasOwnProperty('node_port')) {

      var node_ip = drupalSettings.openlucius_timetracker.node_ip;
      var node_port = drupalSettings.openlucius_timetracker.node_port;

      // Connect to the nodeJs server.
      var socket = io.connect('http://' + drupalSettings.openlucius_timetracker.node_ip + ':' + drupalSettings.openlucius_timetracker.node_port);
      // Temporary logging.
      console.log('Connected to: ' + 'http://' + node_ip + ':' + node_port);
    }
  }
  // The timetracker jquery.
  Drupal.behaviors.timetracker = {
    attach: function (context, settings) {

      // When page is loaded connect to the channel.
      $(document).ready(function () {
        initTimer();
        // Check on refresh if there is an active timer.
        socket.emit('client refresh timer', getCurrentUserId());
        // Get the stopwatch button.
        var btn = $('.timetracker-start-stop-button');

        // Check the button.
        if (btn.length > 0) {
          // Add a trigger.
          btn.off.on('click', function () {
            // TODO: fix the location
            socket.emit('client start timer', getCurrentNodeId(), getCurrentUserId(),
              new Date().getTime(), getCurrentNodeTitle());
            console.log('test');
            // timer = window.setInterval(updateTimer, 1000);
            toggleTimerIcon();
          });
        }
      });
    }
  };

  function startTimerFromRefresh(entry) {

    initActiveTimer(entry);
    toggleTimerIcon();

    timer = window.setInterval(updateTimer, 1000);
  }

  function updateTimer() {

    var timerTimeField = getTimerTimeElement();

    totalActiveTime++;
    // Return the formatted time.
    var formattedTime = moment.unix(totalActiveTime).format("HH:mm:ss");

    // Update the time field.
    timerTimeField.html(formattedTime);

  }

  /**
   * Finish and save a time entry.
   *
   * @param nid
   *   The time entry node id.
   * @param uid
   *   The time entry user id.
   * @param start
   *   The time entry start timestamp.
   * @param stop
   *   The time entry stop timestamp.
   * @param {string} message
   *   The time entry message.
   */
  function finishTimeEntry(nid, uid, start, message) {
    var stop = getCurrentTime();

    $.ajax({
      type: 'GET',
      contentType: "application/json",
      url: drupalSettings.basepath + 'create-time-entry',
      dataType: 'json',
      data: {
        _format: 'json',
        nid: nid,
        uid: uid,
        start: start,
        stop: stop,
        message: message
      },
      success: function (data) {
        /*response(data);*/
        console.log('Posted data.');
      }
    })
  }

  /**
   * Toggle the timer icon.
   */
  function toggleTimerIcon() {
    var btn = $('.timetracker-start-stop-button');
    if (btn.hasClass('fa-play')) {
      btn.removeClass('fa-play').addClass('fa-stop');
    }
    else {
      window.clearInterval(timer);
      btn.removeClass('fa-stop').addClass('fa-play');
      console.log('test');
      socket.emit('client stop timer', getCurrentNodeId(), getCurrentUserId());
    }
  }


  function initTimer() {
    // Get the timer title.
    var timerTitleField = getTimerTitleElement();

    var time = getTimerTimeElement().attr('data-timestamp')
    // Set the correct title of the time entry.
    timerTitleField.html(getCurrentNodeTitle());

    // TODO: fix this / INTERVAL when sending timestamp.
    totalActiveTime = time;
    // Return the formatted time.
    var formattedTime = moment.unix(time).format("HH:mm:ss");

    getTimerTimeElement().html(formattedTime)
  }

  function initActiveTimer(entry) {

    // Reset totalActive time because initTimer.
    totalActiveTime = 0;

    // Get the timer title.
    var timerTitleField = getTimerTitleElement();

    // Set the correct title of the time entry.
    timerTitleField.html(entry.title);

    totalActiveTime = getCurrentTime() - entry.start;

    totalActiveTime += getTimerTimeElement().attr('data-timestamp');

    // Return the formatted time.
    var formattedTime = moment.unix(totalActiveTime).format("HH:mm:ss");

    getTimerTimeElement().html(formattedTime);
  }

  /** ============= getters * =================  */


  /**
   * Get the current time.
   */
  function getCurrentTime() {
    return new Date().getTime() / 1000;
  }

  /**
   * Get the timer time element.
   *
   * @returns {*|HTMLElement}
   */
  function getTimerTimeElement() {
    return $('.time-entry-time');
  }

  /**
   * Get the timer title element.
   *
   * @returns {*|HTMLElement}
   */
  function getTimerTitleElement() {
    return $('.time-entry-title');
  }

  /**
   * Get the current node id from the drupalSettings.
   *
   * @returns {*}
   *   Returns the node id if it has it in the drupalSettings.
   */
  function getCurrentNodeId() {
    if (drupalSettings.hasOwnProperty('openlucius_timetracker')) {
      if (drupalSettings.openlucius_timetracker.hasOwnProperty('nid')) {
        return drupalSettings.openlucius_timetracker.nid;
      }
    }

    // Return false if no node id was found.
    return false;
  }

  /**
   * Get the current node title from the drupalSettings.
   *
   * @returns {*}
   *   Returns the node title if it has it in the drupalSettings.
   */
  function getCurrentNodeTitle() {
    if (drupalSettings.hasOwnProperty('openlucius_timetracker')) {
      if (drupalSettings.openlucius_timetracker.hasOwnProperty('title')) {
        return drupalSettings.openlucius_timetracker.title[0].name;
      }
    }

    // Return false if no node id was found.
    return false;
  }

  /**
   * Get the current user id from the drupalSettings.
   *
   * @returns {*}
   *   Returns the user id if it has it in the drupalSettings.
   */
  function getCurrentUserId() {
    if (drupalSettings.hasOwnProperty('openlucius_timetracker')) {
      if (drupalSettings.openlucius_timetracker.hasOwnProperty('uid')) {
        return drupalSettings.openlucius_timetracker.uid;
      }
    }

    // Return false if no user id was found.
    return false;
  }

  /** =============  NodeJS Listeners=============   */

  socket.on('server refresh timer', function (entry) {
    startTimerFromRefresh(entry);
  });

  socket.on('server stop time', function (entry) {
    finishTimeEntry(entry.cid, entry.uid, entry.start, entry.title);
  });

}(jQuery, Drupal, drupalSettings));
