<?php

namespace Drupal\openlucius_timetracker\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class TimeEntriesController.
 *
 * @package Drupal\openlucius_timetracker\Controller
 */
class TimeEntriesController extends ControllerBase {

  /**
   * Get page content.
   *
   * @param int $uid
   *   The user id from the URL.
   * @param int $from
   *   The from timestamp.
   * @param int $to
   *   The to timestamp.
   *
   * @return string
   *   Returns the page output as html render array.
   */
  public function getPageContent($uid, $from, $to) {

    // Get the services.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');
    $formBuilder = \Drupal::formBuilder();
    $renderService = \Drupal::service('renderer');

    // Get the day totals.
    $day_totals = $timetrackerService->getDayTotals($uid, $from, $to);

    // Get the total time.
    $total = $timetrackerService->getUserTotals($uid, $from, $to);
    $total_time = $this->t('Total: @total hours', ['@total' => $total]);

    // Get the filter form.
    $form = $formBuilder->getForm('Drupal\openlucius_timetracker\Form\TimetrackerFilterForm');
    $filter_form = $renderService->render($form);

    $build = [
      '#theme' => 'timetracker_entries_page',
      '#vars' => [
        'uid' => $uid,
        'total' => $total_time,
        'filter_form' => $filter_form,
      ],
      '#attached' => [
        'library' => [
          'openlucius_timetracker/timetracker_entries',
        ],
        'drupalSettings' => [
          'openlucius_timetracker' => [
            'uid' => $uid,
            'data' => $day_totals,
          ],
        ],
      ],
      // Add the cache tags for invalidating when starting / stopping the timer.
      // https://drupal.stackexchange.com/a/216833/24883
      // $tags = ['read_time_entry_:' . $nid];
      // Cache::invalidateTags($tags);
      'tags' => [
        'user_time_entries_' . $uid,
      ],
    ];

    return $build;
  }

  /**
   * Redirect to the appropriate time entries page.
   */
  public function redirect() {

    // Get the services.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');

    // Set the arguments.
    $uid = \Drupal::currentUser()->id();

    // Get the first second of today.
    $today = $timetrackerService->generateTimestamp();

    // Get the entries for the user.
    $from = $today['start_month'];
    $to = $today['end_month'];

    // Create the url by filling in the route and arguments.
    $redirect_url = Url::fromRoute('openlucius_timetracker.time_entries_page', [
        'uid' => $uid,
        'from' => $from,
        'to' => $to,
      ]
    );

    // Create and return the redirect response.
    $response = new RedirectResponse($redirect_url->toString());
    return $response->send();
  }

}
