<?php

namespace Drupal\openlucius_timetracker\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Class TimeTrackerController.
 *
 * @package Drupal\openlucius_timetracker\Controller
 */
class TimetrackerController extends ControllerBase {

  protected $timetrackerService;

  /**
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
   *
   * @return static
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('openlucius_timetracker.timetracker_service')
    );
  }

  /**
   * TimetrackerController constructor.
   *
   * @param $timetrackerService
   *   The timetracker service.
   */
  public function __construct($timetrackerService) {
    $this->timetrackerService = $timetrackerService;
  }

  /**
   * Create a time entry.
   */
  public function createTimeEntry() {

    $request = \Drupal::request();

    // Get the posted data.
    $nid = $request->get('nid');
    $uid = $request->get('uid');
    $start = $request->get('start');
    $stop = $request->get('stop');
    $message = $request->get('message');

    \Drupal::logger('timetracker')
      ->notice('Entry values: uid: %uid, nid: %nid, start: %start, stop: %stop, message: %message.', [
        '%nid' => $nid,
        '%uid' => $uid,
        '%start' => $start,
        '%stop' => $stop,
        '%message' => $message,
      ]);

    // Create the entry.
    //$entry = \Drupal::service('openlucius_timetracker.timetracker_service')->createTimeEntry($uid, $nid);
    $entry = $this->timetrackerService->createTimeEntry($nid, $uid, $start, $stop, $message);

    // Always return a response.
    return new JsonResponse([
      'entry' => $entry,
    ]);
  }

  /**
   * Read a time entry.
   *
   * @return array
   *   Returns the entry as array.
   */
  public function readTimeEntry() {

    $request = \Drupal::request();

    // Get the posted data.
    $nid = $request->get('nid');
    $uid = $request->get('uid');

    // Create the entry.
    $entry = $this->timetrackerService->readTimeEntry($nid, $uid);

    // Always return a response.
    return new JsonResponse([
      'entry' => $entry,
    ]);
  }

  /**
   * Create a time entry.
   */
  public function updateTimeEntry() {

    // Get the parameters.
    $uid = \GuzzleHttp\json_decode($_GET['uid']);
    $nid = \GuzzleHttp\json_decode($_GET['nid']);

    // Create the entry.
    //$entry = \Drupal::service('openlucius_timetracker.timetracker_service')->updateTimeEntry($uid, $nid);
    $entry = $this->timetrackerService->updateTimeEntry($uid, $nid);

    // Always return a response.
    return new JsonResponse([
      'entry' => $entry,
    ]);
  }

  /**
   * Create a time entry.
   */
  public function deleteTimeEntry() {

    // Get the parameters.
    $uid = \GuzzleHttp\json_decode($_GET['uid']);
    $nid = \GuzzleHttp\json_decode($_GET['nid']);

    // Create the entry.
    //$entry = \Drupal::service('openlucius_timetracker.timetracker_service')->deleteTimeEntry($uid, $nid);
    $entry = $this->timetrackerService->deleteTimeEntry($uid, $nid);

    // Always return a response.
    return new JsonResponse([
      'entry' => $entry,
    ]);
  }
}
