<?php

namespace Drupal\openlucius_timetracker\Form;

use Drupal\Core\Datetime\DrupalDateTime;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class TimetrackerFilterForm.
 *
 * @package Drupal\openlucius_timetracker\Form
 */
class TimetrackerFilterForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'timetracker_filter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    //db_truncate('watchdog')->execute();

    // Get the services.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');

    // Get the current route.
    $route = \Drupal::routeMatch();
    $uid = (int) $route->getParameter('uid');
    $from = (int) $route->getParameter('from');
    $to = (int) $route->getParameter('to');

    $form['uid'] = [
      '#type' => 'select',
      '#title' => $this->t('User'),
      '#options' => $timetrackerService->getUsers(),
      '#default_value' => $uid,
      '#prefix' => '<div class="form-group col-lg-12">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
        ],
      ],
    ];
    $form['from'] = [
      '#type' => 'datetime',
      '#title' => $this->t('From'),
      '#default_value' => DrupalDateTime::createFromTimestamp($from),
      '#prefix' => '<div class="form-group col-lg-12">',
      '#suffix' => '</div>',
      /*'#attributes' => [
        'class' => [
          'form-control',
        ],
      ],*/
    ];
    $form['to'] = [
      '#type' => 'datetime',
      '#title' => $this->t('To'),
      '#default_value' => DrupalDateTime::createFromTimestamp($to),
      '#prefix' => '<div class="form-group col-lg-12">',
      '#suffix' => '</div>',
      /*'#attributes' => [
        'class' => [
          'form-control',
        ],
      ],*/
    ];
    $form['filter'] = [
      '#type' => 'submit',
      '#value' => $this->t('Filter'),
      '#prefix' => '<div class="form-group col-lg-12">',
      '#suffix' => '</div>',
      '#attributes' => [
        'class' => [
          'form-control',
          'btn-success',
          'add-time-entry-btn',
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Get the values.
    $uid = $form_state->getValue('uid');
    $from = $form_state->getValue('from');
    $to = $form_state->getValue('to');

    // Create the url by filling in the route and arguments.
    $redirect_url = Url::fromRoute('openlucius_timetracker.time_entries_page', [
        'uid' => $uid,
        'from' => $from->getTimestamp(),
        'to' => $to->getTimestamp(),
      ]
    );

    // Redirect with a redirect response.
    $response = new RedirectResponse($redirect_url->toString());
    $response->send();
  }

}
