<?php

namespace Drupal\openlucius_timetracker\Plugin\Block;

use Drupal\Core\Block\BlockBase;

/**
 * Provides a 'TimetrackerBlock' block.
 *
 * @Block(
 *  id = "timetracker_block",
 *  admin_label = @Translation("Timetracker block"),
 * )
 */
class TimetrackerBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {

    // Get the services.
    $timetrackerService = \Drupal::service('openlucius_timetracker.timetracker_service');
    $timetrackerTableService = \Drupal::service('openlucius_timetracker.timetracker_table_service');

    $formBuilder = \Drupal::formBuilder();
    $renderService = \Drupal::service('renderer');

    // Get the user id, node id and current time.
    $uid = \Drupal::currentUser()->id();
    $nid = $timetrackerService->getCurrentNodeId();

    // Process variables for the build.
    $time_entries = $timetrackerTableService->getTimeEntriesTable($nid);
    $form = $formBuilder->getForm('Drupal\openlucius_timetracker\Form\TimetrackerForm');
    $add_time_form = $renderService->render($form);

    // Get the total time.
    $total_time = $timetrackerService->getFormattedTotalTime($nid);
    $modal_title = t('Total tracked time: @total', ['@total' => $total_time]);

    // Get the config.
    $config = \Drupal::config('openlucius_config.settings');

    // Get the current node title.
    $title = get_channel_name($nid);
    $title = $title[0]->name;

    $build = [
      '#theme' => 'timetracker',
      '#vars' => [
        'nid' => $nid,
        'time_entries' => $time_entries,
        'add_time_form' => $add_time_form,
        'modal_title' => $modal_title,
        'my_time' => $timetrackerService->getMyTime($nid, $uid, 'formatted'),
        'my_timestamp' => $timetrackerService->getMyTime($nid, $uid),
      ],
      '#attached' => [
        'library' => [
          'openlucius_timetracker/timetracker',
        ],
        'drupalSettings' => [
          'openlucius_timetracker' => [
            'uid' => $uid,
            'nid' => $timetrackerService->getCurrentNodeId(),
            'title' => $title,
            'my_time' => $timetrackerService->getMyTime($nid, $uid, 'formatted'),
            'node_ip' => $config->get('openlucius_nodejs_ip'),
            'node_port' => $config->get('openlucius_nodejs_port'),
          ],
        ],
      ],
      '#cache' => [
        'max-age' => 0,
      ],
      /*Add the cache tags for invalidating when starting / stopping the timer.
      https://drupal.stackexchange.com/a/216833/24883
      $tags = ['read_time_entry_:' . $nid];
      Cache::invalidateTags($tags);
      'tags' => [
        'read_time_entry_' . $nid,
      ],*/
    ];

    return $build;
  }

}
