(function ($, drupalSettings) {
  'use strict';
  // Connect to the nodeJs server.
  var socket = io.connect('//' + drupalSettings.nodeIp + ':' + drupalSettings.nodePort);

  const IS_TYPING_TIMEOUT = 3000;
  const MESSAGE_TYPE_TEXT = 0;
  const MESSAGE_TYPE_IMAGE = 1;
  const MESSAGE_TYPE_OTHER = 2;
  // Get the channel id.
  var cId = getChannelId();

  // Set history variables.
  var loadedItems = 0;
  var prevScrollHeight = 0;

  // Retrieve the uId and user name and user image link from the drupalSettings.
  var uId = drupalSettings.userId;
  var uName = drupalSettings.userName;
  var userImage = drupalSettings.userImage;

  // Mentioning variables.
  var isMention = false;
  var triggered = false;
  var mentionCharacter = '@';
  var mentionMsg = '';
  var mentions = [];

  // When page is loaded connect to the channel.
  $(document).ready(function () {
    readMentions();
    connectToRoom();
    getHistory();
    checkNewMessages();
    getMentions();
    $('#chat').focus();

    /* Start of mentioning logic */
    $('#chat').autocomplete({
      minLength: 0,
      source: function (request, response) {
        // Make sure to get only the text after the mention character.
        request = request.term.substr(request.term.lastIndexOf("@") + 1);
        $.ajax({
          type: 'GET',
          contentType: "application/json",
          url: drupalSettings.basepath + 'get_channel_users',
          dataType: 'json',
          data: {
            _format: 'json',
            cid: cId,
            input: request
          },
          success: function (data) {
            response(data);
          }
        });
      },
      search: function (event, ui) {
        if (!isMention) {
          event.preventDefault();
          $('#chat').autocomplete('close');
        }

      },
      focus: function (event, ui) {
        event.preventDefault();
      },
      select: function (event, ui) {
        event.preventDefault();
        mentionMsg = mentionMsg.substr(0, mentionMsg.lastIndexOf("@") + 1);
        mentionMsg = mentionMsg + ui.item.label;
        $('#chat').val(mentionMsg + ' ');
        mentions.push(ui);
        isMention = false;
      },
      open: function (event, ui) {
        mentionMsg = $('#chat').val();
      },
      position: {my: "left bottom", at: "left top", collision: "flip"}
    });
    /* End of mention logic */

    /* File upload logic */

    Dropzone.options.uploadWidget = {
      maxFilesize: 500,
      addRemoveLinks: true,
      dictResponseError: 'Server not Configured',
      clickable: '.dz-clickable',
      dictDefaultMessage: '',
      // TODO: set accepted files.
      // acceptedFiles: ".png,.jpg,.gif,.bmp,.jpeg",
      init: function () {
        var self = this;
        // config
        self.options.addRemoveLinks = true;
        self.options.dictRemoveFile = "Delete";
// TODO: check for pdf
// TODO: check for zip or other
// TODO: check for image
        self.on('sending', function (file, xhr, formData) {
          formData.append('cid', cId);
        });

        self.on('success', function (file, response) {

        })
      },
      success: function (file, response) {
        file.previewElement.remove();
        //todo: CHECK OTHER FILE TYPES!
        if (file.type.indexOf('image') !== -1) {
          sendMessage(response.filePath, MESSAGE_TYPE_IMAGE);
        }
        else {
          sendMessage(response.filePath, MESSAGE_TYPE_OTHER);
        }
      }


    };

    var lastTarget = null;

    function isFile(evt) {
      var dt = evt.dataTransfer;

      for (var i = 0; i < dt.types.length; i++) {
        if (dt.types[i] === "Files") {
          return true;
        }
      }
      return false;
    }

    document.getElementById("upload-widget").addEventListener("dragenter", function (e) {
      if (isFile(e)) {
        lastTarget = e.target;
        document.querySelector("#dropzone").style.display = "";
        document.querySelector("#dropzone").style.opacity = 1;
        document.querySelector("#textnode").style.fontSize = "48px";
      }
    });

    document.getElementById("upload-widget").addEventListener("dragleave", function (e) {
      e.preventDefault();
      if (e.target === lastTarget) {
        document.querySelector("#dropzone").style.display = "none";
        document.querySelector("#dropzone").style.opacity = 0;
        document.querySelector("#textnode").style.fontSize = "42px";
      }
    });

    document.getElementById("upload-widget").addEventListener("dragover", function (e) {
      e.preventDefault();
    });

    document.getElementById("upload-widget").addEventListener("drop", function (e) {
      e.preventDefault();
      document.querySelector("#dropzone").style.display = "none";
      document.querySelector("#dropzone").style.opacity = 0;
      document.querySelector("#textnode").style.fontSize = "42px";
    });

  });

  $('#chat_messages').on('scroll', function () {
    if ($(this).scrollTop() === 0) {
      getHistory();
    }
    else {
      return;
    }
  });

  /**
   * Send message to server.
   * @param msg
   * @param type
   */
  function sendMessage(msg, type) {
    // Create data object. Other data will be added on the server side.

    var data = {
      uname: uName,
      image: userImage,
      body: msg,
      cid: cId,
      type: type,
      date: new Date()
    };
    socket.emit('client message', data);
    checkNewMessages();
  }

  /**
   * Join a channel.
   */
  function connectToRoom() {
    var data = {
      uid: uId,
      cid: cId,
      uname: uName
    };
    socket.emit('join room', data);
  }

  /**
   * Leave a channel.
   * @param uid
   * @param cid
   */
  function leaveRoom(uid, cid) {

    var data = {
      uid: uid,
      cid: cid
    };
    socket.emit('leave room', data);
  }

  window.leaveRoom = leaveRoom;
  window.userAdd = userAdd;

// Receive messages from others and show them to the user.
  socket.on('server message', function (data) {
    switch (data.type) {
      case 0 :
        addTextMessage(false, data);
        break;
      case 1:
        addImageMessage(false, data);
        // image message here
        break;
      case 2:
        addFileMessage(false, data);
        // image message here
        break;

    }
    scrollToBottom(false);
  });


  socket.on('server disable input', function () {
    document.getElementById('chat').disabled = true;
  });

  // Receve messages from others and show them to the user.
  socket.on('server chat history', function (data) {
    if (data.length !== 0) {

      data.forEach(function (msg) {

        if (msg.type === MESSAGE_TYPE_TEXT) {
          addTextMessage(true, msg);
        }
        else {
          if (msg.type === MESSAGE_TYPE_IMAGE) {
            addImageMessage(true, msg);
          }
          else {
            addFileMessage(true, msg);
          }
        }
      });
      // This variable should be the same as the limit of the nodejs server query.
      loadedItems += 20;
      scrollToBottom(true);
    }
  });

  // Listen for new messages in other channels that this user is in.
  socket.on('new message', function (cid) {

    // Check if we are not in the active channel or ticket.
    if (!($('#' + cid).hasClass('active')) && !$('task_id_' + cid).hasClass('active')) {
      // Is the new message in a channel?
      if ($('#' + cid).length) {
        $('#channel_id_left_sidebar_' + cid).addClass('new-message');
        // get the parent.
        document.getElementById("reddot").style.display = "block";
      }
      // The new message is in a ticket.
      else {
        if (cid != cId) {
          $('#channel_id_left_sidebar_' + cid).addClass('new-message');
          // Get the id of the parent Channel!
          var parentChannelId = $('#task_id_' + cid).parent().parent().attr('id');
          $('#channel_id_left_sidebar_' + parentChannelId).addClass('new-message');
          // var element = document.getElementById(id);
          // console.log(element);
        }
      }
    }
    // If we are in the channel the new message is in let the server know.
    if (cid == cId) {
      socket.emit('client in channel', uId, cId);
    }
  });

  // Retreive all the channels the user has unread messages in.
  socket.on('server refresh new message', function (channels) {
    channels.forEach(function (channel) {
      if (channel.new_message) {
        if (!($('#channel_id_left_sidebar_' + channel.cid).hasClass('active'))) {
          $('#channel_id_left_sidebar_' + channel.cid).addClass('new-message');
          $('#channel_id_left_sidebar_' + channel.cid).addClass('new-message');
        }
        if (channel.cid == cId) {
          $('#channel_id_left_sidebar_' + channel.cid).removeClass('new-message');
        }
      }
    });
  });
  // Is typing call from server.
  socket.on('server is typing', function (uname) {
    var div = document.getElementById('is-typing');
    if (div.innerHTML.indexOf(uname) == -1) {
      if (uname !== uName) {
        if (div.hasChildNodes()) {
          div.innerHTML = uname + ', ' + div.innerHTML;
        }
        else {
          div.innerHTML = div.innerHTML + uname + ' is typing...';
        }
      }
    }
  });

  // Stopped typing call from server.
  socket.on('server stopped typing', function (uname) {
    var div = document.getElementById('is-typing');
    if (div.innerHTML.indexOf(',') >= 0) {
      div.innerHTML = div.innerHTML.replace(uname + ',', '');
    }
    else {
      div.innerHTML = '';
    }
  });

  socket.on('server mention', function (cid, mentions) {
    var li = $('#' + cid + ' a');
    var task = $('#task_id_' + cid + ' a');
    if (cid !== cId) {
      // TODO: investigate bug why mentions inc on server still returns 0  when 0 mentions.
      mentions++;
      $('#mentioned' + cid).remove();
      li.append('<small class="label pull-right bg-red" id="mentioned' + cid + '">' + mentions + '</small>');
      task.append('<small class="label pull-right bg-red" id="mentioned' + cid + '">' + mentions + '</small>');
    }
    else {
      socket.emit('client read mentions', uId, cId)
    }

  });

  socket.on('server request mentions', function (mentions) {
    mentions.forEach(function (mention) {
      if (mention.cid !== cId && mention.mentions !== 0) {
        var li = $('#' + mention.cid + ' a');
        var task = $('#task_id_' + mention.cid + ' a');
        $('#mentioned' + mention.cid).remove();
        li.append('<small class="label pull-right bg-red" id="mentioned' + mention.cid + '">' + mention.mentions + '</small>');
        task.append('<small class="label pull-right bg-red" id="mentioned' + mention.cid + '">' + mention.mentions + '</small>');
      }
    });
  });

  // Code to add incoming messages to the HTML.
  function addTextMessage(isHistory, data) {
    var hours = getHours(data.date);
    var minutes = getMinutes(data.date);
    var day = getDay(data.date);
    var month = getMonth(data.date);
    if (data.cid == cId) {

      if (isHistory) {
        $('.msgs')
          .prepend('<div class="box-comment"><img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;"><div class="comment-text"><span class="username">' + data.uname + '<span class="text-muted pull-right"><i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '</span> </span>' + data.body + '</div></div>');
      }
      else {
        $('.msgs')
          .append('<div class="box-comment"><img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;"><div class="comment-text"><span class="username">' + data.uname + '<span class="text-muted pull-right"><i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '</span> </span>' + data.body + '</div></div>');
      }
    }
  }

  function addImageMessage(isHistory, data) {
    var hours = getHours(data.date);
    var minutes = getMinutes(data.date);
    var day = getDay(data.date);
    var month = getMonth(data.date);
    if (data.cid == cId) {
      var filename = getNameFromUri(data.body);
      if (isHistory) {
        $('.msgs')
          .prepend(
            '<div class="box-comment">' +
            '<img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;">' +
            '<div class="comment-text">' +
            '<span class="username">' + data.uname + '' +
            '<span class="text-muted pull-right">' +
            '<i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '' +
            '</span>' +
            '</span>' +
            '<a class="openlucius-attachment" target="_blank" href="' + data.body + '">' +
            '<img class="image-message" src="' + data.body + '">' +
            '<div class="col-md-8 wrap-word file-name">' + filename + '</div>' +
            '</a>' +
            '</div>' +
            '</div>'
          );
      }
      else {
        $('.msgs')
          .append(
            '<div class="box-comment">' +
            '<img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;">' +
            '<div class="comment-text">' +
            '<span class="username">' + data.uname + '' +
            '<span class="text-muted pull-right">' +
            '<i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '' +
            '</span>' +
            '</span>' +
            '<a class="openlucius-attachment" target="_blank" href="' + data.body + '">' +
            '<img class="image-message" src="' + data.body + '">' +
            '<div class="col-md-8 wrap-word file-name">' + filename + '</div>' +
            '</a>' +
            '</div>' +
            '</div>'
          );
      }
    }
  }


  function addFileMessage(isHistory, data) {
    var hours = getHours(data.date);
    var minutes = getMinutes(data.date);
    var day = getDay(data.date);
    var month = getMonth(data.date);
    if (data.cid == cId) {
      var filename = getNameFromUri(data.body);
      var defaultFile = drupalSettings.basepath + 'profiles/openlucius/themes/openluciustheme/assets/images/default-file.png';

      if (isHistory) {
        $('.msgs')
          .prepend('' +
            '<div class="box-comment">' +
            '<img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;">' +
            '<div class="comment-text">' +
            '<span class="username">' + data.uname + '' +
            '<span class="text-muted pull-right">' +
            '<i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '' +
            '</span>' +
            '</span>' +
            '<a class="openlucius-attachment file-attachment" target="_blank" href="' + data.body + '">' +
            '<i class="fa fa-file-text image-message"></i>' +
            '<div class="col-md-8 wrap-word file-name">' + filename + '</div>' +
            '</a>' +
            '</div>' +
            '</div>'
          );
      }
      else {
        $('.msgs')
          .append('' +
            '<div class="box-comment">' +
            '<img class="img-circle img-sm hoverZoomLink" src="' + data.image + '"alt="&nbsp;">' +
            '<div class="comment-text">' +
            '<span class="username">' + data.uname + '' +
            '<span class="text-muted pull-right">' +
            '<i class="fa fa-clock-o"></i>&nbsp;' + day + '&nbsp;' + month + '&nbsp;' + hours + ':' + minutes + '' +
            '</span>' +
            '</span>' +
            '<a class="openlucius-attachment file-attachment" target="_blank" href="' + data.body + '">' +
            '<i class="fa fa-file-text image-message"></i>' +
            '<div class="col-md-8 wrap-word file-name">' + filename + '</div>' +
            '</a>' +
            '</div>' +
            '</div>'
          );
      }
    }
  }

  // Switch case to catch keydown events.
  document.getElementById('chat').addEventListener("keydown", function (e) {

    if (e.keyCode == 13 && !e.shiftKey) {
      e.preventDefault();
      // Check for empty message.
      if ($('#chat').val().charAt(0) != ' ' && $('#chat').val().charAt(0) != '') {
        if (!isMention) {
          var value = $('#chat').val().replace(/\n/g, '<br/>');
          value = filterXSS(linkify(value));
          sendMessage(value, MESSAGE_TYPE_TEXT);
          if (mentions) {
            mentions.forEach(function (mention) {
              if ($('#chat').val().indexOf(mention.item.value) >= 0) {
                setTimeout(socket.emit('client mention', mention.item.id,
                  cId), 500);
              }
            });
            mentions = [];
          }
        }
        $('#chat').val('');
      }
    }
  });

  // Get the send button.
  var btn = $('.button-wrapper .send-message');

  // Check the send button.
  if (btn.length > 0) {

    // Add the click event.
    btn.on('click', function () {
      sendMessage($('#chat').val(), MESSAGE_TYPE_TEXT);
    })
  }

  /* logic to implement is typing function */

  var typing = false;
  var timeout = undefined;

  function timeoutFunction() {
    typing = false;
    socket.emit('client stopped typing', uName, cId);
  }

  document.getElementById('chat').addEventListener("keyup", function (e) {
    // Get the character -1 of the cursor.
    var prevCharacter = $('#chat')
      .val()
      .charAt($('#chat').val().lastIndexOf(mentionCharacter) - 1);
// TODO: make a switch case of this.
    if (e.keyCode == 13) {
      // Do nothing.
    }
    else {
      if (typing === false) {
        typing = true;
        socket.emit('client is typing', uName, cId);
        timeout = setTimeout(timeoutFunction, IS_TYPING_TIMEOUT);
        clearTimeout(timeout);
        timeout = setTimeout(timeoutFunction, IS_TYPING_TIMEOUT);
      }
    }

    // Code that checks if the mention functions should be triggered.
    if (e.keyCode == 50) {
      // Make sure it is an '@' and not a '2'.
      if ($('#chat').val().indexOf('@') !== -1) {
        // Check if it is a real mention and not i.e. an email address.
        if (prevCharacter === ' ' || prevCharacter === '') {
          isMention = true;
        }
      }
    }

    // Check if mentioning is canceled
    if ($('#chat').val().indexOf('@') === -1) {
      isMention = false;
    }
    if ($('#chat')
        .val()
        .indexOf('@') !== -1 &&
      (prevCharacter !== ' ' && prevCharacter !== '')) {
      isMention = false;
    }
    if (e.keyCode === 32) {
      isMention = false;
    }
  });

  /* End of is typing logic */
  function getNameFromUri(uri) {
    return uri.substring((uri.lastIndexOf('/') + 1), uri.lastIndex);
  }

  function getHours(oldDate) {
    // Format library wants a date object so we create a date object.
    return moment(oldDate).format('H');
  }

  function getMinutes(oldDate) {
    // Format library wants a date object so we create a date object.
    return moment(oldDate).format('mm');
  }

  function getDay(oldDate) {
    // Format library wants a date object so we create a date object.
    return moment(oldDate).format('DD');
  }

  function getMonth(oldDate) {
    // Format library wants a date object so we create a date object.
    return moment(oldDate).format('MMM');
  }

  function scrollToBottom(isHistory) {
    var box = document.getElementById("chat_messages");
    if (isHistory) {
      box.scrollTop = (box.scrollHeight - prevScrollHeight);
    }
    else {
      box.scrollTop = box.scrollHeight;
    }
    prevScrollHeight = box.scrollHeight;
  }

  function checkNewMessages() {
    socket.emit('client refresh new message', uId);
  }

  // Let server know mentions are read.
  function readMentions() {
    $('#mentioned' + cId).remove();
    socket.emit('client read mentions', uId, cId);
  }

  // Check on refresh if we have mentions
  function getMentions() {
    socket.emit('client request mentions', uId);
  }

  function getHistory() {
    socket.emit('client chat history', cId, loadedItems);
  }

  function userAdd(uname, cid) {
    socket.emit('client user add', uname, cid);
  }

  /**
   * @returns {int}
   * Method that retreives the channel id from the url
   */
  function getChannelId() {
    // Get the url.
    var url = window.location.href;
    // Split the url with the "/".
    var parts = url.split("/");
    // Initialize id variable.
    var id;
    // Loop trough the array and get the value that comes after the channel(always channel id).
    parts.forEach(function (part, index) {
      if (part === "channel") {
        id = parts[index + 1];
      }
    });
    return id;
  }

  function linkify(text) {
    var urlRegex = /(((https?:\/\/)|(www\.))[^\s]+)/g;
    return text.replace(urlRegex, function (url, b, c) {
      var url2 = (c == 'www.') ? 'https://' + url : url;
      return '<a href="' + url2 + '" target="_blank">' + url + '</a>';
    })
  }
}(jQuery, drupalSettings));
