<?php

namespace Drupal\openlucius_channel\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Database;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\openlucius_channel\Entity\Channel;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides route responses for the Example module.
 */
class ChannelController extends ControllerBase {

  /**
   * Returns a page with archived channel.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getArchivedChannelsContent() {

    global $base_path;

    // Get current user id
    $uid = \Drupal::currentUser()->id();

    // Get Channels data
    $query = \Drupal::database()->select('ol_user_channel', 'ouc');
    $query->fields('ouc', ['cid']);
    $query->addField('cfd', 'name');
    $query->addField('cfd', 'type');
    $query->join('channel_field_data', 'cfd', 'ouc.cid = cfd.id');
    $query->condition('ouc.uid', $uid, '=');
    $query->condition('cfd.status', 0);
    $query->orderBy('cfd.name', 'ASC');
    $channels = $query->execute()->fetchAll();

    // Initialize arrays
    $channeldata = array();
    $dmdata = array();

    // Check if there are results
    if (isset($channels)) {

      // Build array for Channels
      foreach ($channels as $channel) {

        // Build array for Channels
        if ($channel->type == '1') {
          $channeldata[] = array(
            'cid'   => $channel->cid,
            'name' => $channel->name,
          );
        }

        // Build array for Private Messages
        if ($channel->type == '2') {

          $display_names = array();

          // Query names of 'others' in this private message
          $query = \Drupal::database()->select('ol_user_channel', 'ouc');
          $query->addField('ufd', 'name');
          $query->join('users_field_data', 'ufd', 'ouc.uid = ufd.uid');
          $query->condition('ouc.uid', $uid, '!=');
          $query->condition('ouc.cid', $channel->cid, '=');
          $query->orderBy('ufd.name', 'ASC');
          $names = $query->execute()->fetchAll();

          foreach ($names as $name) {
            $display_names[] = $name;
          }

          // Exception for 'user own' private channel
          if(empty($display_names)){
            $display_names[]->name = "Me";
          }

          $dmdata[] = array(
            'cid' => $channel->cid,
            'name' => $display_names[0]->name,
          );
        }
        // Build array for Tasks
        if ($channel->type == '3') {

          //print $channel->cid;

          // Fetch parent Channel
          $query = \Drupal::database()->select('ol_tasks', 'olt');
          $query->addField('cfd', 'name', 'parent_name');
          $query->addField('olt', 'parent_id');
          $query->join('channel_field_data', 'cfd', 'cfd.id = olt.parent_id');
          $query->condition('olt.tid', $channel->cid, '=');
          $parentData = $query->execute()->fetchAssoc();

          $taskdata[] = array(
            'cid'   => $channel->cid,
            'name' => $channel->name,
            'parent_id' => $parentData['parent_id'],
            'parent_name' => $parentData['parent_name'],
          );
        }
      }
    }

    // Misc needed vars.
    $cacheTime = array('max-age' => 0);

    // Build it.
    $build = array(
      '#theme' => 'page_archived_channels',
      '#channels' => $channeldata,
      '#dmsg' => $dmdata,
      '#task_data' => $taskdata,
      '#path' => $base_path,
      '#cache' => $cacheTime,
    );

    return $build;

  }
  /**
   * Returns a page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getPageContent($channelid) {

    // Check channel access, return false if check is false
    if(checkChannelAccess($channelid, 'getPageContent') != TRUE){ return FALSE; }

    global $base_path;

    $form = Url::fromRoute('openlucius_channel.file_upload')->toString();

    // Get the config.
    $config = \Drupal::config('openlucius_config.settings');


    // Load user.
    $uid = \Drupal::currentUser()->id();
    $user = \Drupal::currentUser();
    $channel = \Drupal\openlucius_channel\Entity\Channel::load($channelid);

    $otherUsers = '';
    $getUsers = NULL;

    // Get user image.
    $user = User::load($uid);

    $userImage = getUserImage($user);

    // Retrieves a \Drupal\Core\Database\Connection which is a PDO instance.
    $connection = Database::getConnection();

    // Retrieves a PDOStatement object.
    $sth = $connection->select('ol_user_channel', 'x')
      ->fields('x')
      ->condition('uid', $uid, '=')
      ->condition('cid', $channelid, '=');

    // Execute the statement.
    $executed = $sth->execute();
    // Get all the results.
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);
    if (!$results == array()) {
    }
    else {
      $response = new RedirectResponse($base_path . "home");
      $response->send();
    }
    if ($channel != NULL) {
      $getUsers = $channel->getUsers();
    }
    else {
      $response = new RedirectResponse($base_path . "home");
      $response->send();
    }

    if ($getUsers != '') {
      foreach ($getUsers as $otherUser) {
        $otherUsers .= '<li>' . $otherUser->uid . '</li>';
      }
    }
    $this->getChannelUsers();


    $cacheTime = array('max-age' => 0);
    // The #attached attribute injects objects into the js.
    $element = array(
      '#theme' => 'channel',
      '#nodeIp' => $config->get('openlucius_nodejs_ip'),
      '#nodePort' => $config->get('openlucius_nodejs_port'),
      '#cache' => $cacheTime,
      '#upload' => $form,
      '#attached' => array(
        'library' => array('openlucius_channel/chat'),
        'drupalSettings' => array(
          'userId' => $user->id(),
          'userName' => $user->getAccountName(),
          'userImage' => $userImage,
          'basepath' => $base_path,
          'nodeIp' => $config->get('openlucius_nodejs_ip'),
          'nodePort' => $config->get('openlucius_nodejs_port'),
        ),
      ),
    );

    return $element;
  }

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getUsers($channelid) {

    // Check channel access, return false if check is false
    if(checkChannelAccess($channelid, 'getUsers') != TRUE){ return FALSE; }

    $channel = Channel::load($channelid);
    $otherUsers = '';
    $getUsers = $channel->getUsers();

    if ($getUsers != '') {
      foreach ($getUsers as $otherUser) {
        $otherUsers .= '<li>' . $otherUser->uid . ' Remove </li>';
      }
    }
    $cacheTime = array('max-age' => 0);

    $element = array(
      '#markup' => 'users: ' . $otherUsers,
      '#cache' => $cacheTime,
    );

    return $element;
  }

  /**
   * Ajax function to remove users from channel.
   *
   * @return JsonResponse
   *   A json to be read by frontend.
   */
  public function remove() {

    // Load parameter. Channel id.
    $uid = $_GET['uid'];
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);
    $uid = \GuzzleHttp\json_decode($uid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'remove') != TRUE){ return FALSE; }

    $channelid = Channel::load($cid);
    $channelid->removeUsers($uid);

    // Get 'task channels' that are attached to this 'main channel'
    $query = \Drupal::database()->select('ol_tasks', 'olt');
    $query->addField('olt', 'tid');
    $query->condition('olt.parent_id', $cid, '=');
    $taskChannels = $query->execute()->fetchAll();

    // Remove users from 'Task sub channels'.
    foreach ($taskChannels as $taskChannel) {
      $taskChannelId = Channel::load($taskChannel->tid);
      $taskChannelId->removeUsers($uid);
    };

    $my_array = array(
      'cid' => $cid,
      'uid' => $uid,
    );

    return new JsonResponse($my_array);
  }

  /**
   * Function used by Ajax-call to add users to channel without refresh.
   */
  public function add() {

    global $base_path;

    // Load parameter. Channel id.
    $uname = $_GET['name'];
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'add') != TRUE){ return FALSE; }

    $channel = Channel::load($cid);

    $user = user_load_by_name($uname);
    $uid = $user->id();
    if ($uid) {

      // If user not already exists in that channel, add him.
      if (!$channel->checkSingleUser($uid)) {
        $channel->setUsers($uid, 'sidebar_right');


        // Load newly added user.
        $newUser = User::load($uid);

        // Load image url of newly added users image.
        $fid = $newUser->get('field_user_image')->getValue();
        $newUserImage = $base_path . 'profiles/openlucius/themes/openluciustheme/assets/images/default-user.png';
        if (isset($fid)) {

          $fid = $fid[0]['target_id'];

          $file = File::load($fid);

          if ($file != NULL) {
            $newUserImage = $file->url();
          }
        }


        // Theme data into user_block_item (single user in user_block.
        $toAppend = '<li>
                <img src="' . $newUserImage . '" class="user-image-right" alt="User Image">
                <a class="users-list-name" href="#">' . $newUser->getAccountName() . '</a>
                <i class="fa fa-times remove-button">
                    <input type="hidden" name="uid" class="uid" value="' . $uid . '">
                    <input type="hidden" name="cid" class="cid" value="' . $cid . '">
                </i>
            </li>';

        // Get 'task channels' that are attached to this 'main channel'
        $query = \Drupal::database()->select('ol_tasks', 'olt');
        $query->addField('olt', 'tid');
        $query->condition('olt.parent_id', $cid, '=');
        $taskChannels = $query->execute()->fetchAll();

        // Add user from 'Task sub channels'.
        foreach ($taskChannels as $taskChannel) {
          $taskChannelId = Channel::load($taskChannel->tid);
          $taskChannelId->setUsers($uid);
        };

        $my_array = array(
          'cid' => $cid,
          'uid' => $uid,
          'toAppend' => $toAppend,
        );

        return new JsonResponse($my_array);

      }
      else {
        // 'already in channel'.
        header('HTTP/1.1 500 user already in channel');
        header('Content-Type: application/json; charset=UTF-8');
        die(json_encode(array('message' => 'ERROR', 'code' => 1337)));
      }
    }
    else {
      // 'already in channel'.
      header('HTTP/1.1 500 user does not exist');
      header('Content-Type: application/json; charset=UTF-8');
      die(json_encode(array('message' => 'ERROR', 'code' => 1337)));
    }
  }

  /**
   * Function used by Ajax-call to add users to channel without refresh.
   */

  public function addEmail() {

    // Load parameter. Channel id and email.
    $email = $_GET['emailaddress'];
    $cid = $_GET['cid'];
    // Decode Json object.
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'addEmail') != TRUE){ return FALSE; }

    // Load channel.
    $channel = Channel::load($cid);
    // Validate email
    $valid_email = \Drupal::service('email.validator')->isValid($email);
    // Load potential user.
    $user = user_load_by_mail($email);

    // Not a valid email, so die.
    if (!$valid_email) {
      header('HTTP/1.1 500 email invalid | 9863');
      header('Content-Type: application/json; charset=UTF-8');
      die(json_encode(array('message' => 'ERROR', 'code' => 1337)));
    }

    // Email does not exists, create new user and add it to team and channel.
    if (empty($user)) {
      // Setup new user
      $user = User::create();
      $user->enforceIsNew();
      $user->setEmail($email);
      $user->setUsername($email);
      $user->activate();
      $user->save();
      // Send initial login mail.
      _user_mail_notify('register_no_approval_required', $user);
      // Load new user.
      $user = user_load_by_mail($email);
      $uid = $user->id();
      // Add new user to default team of logged in user and targeted channel.
      addToTeam($uid);
      $channel->setUsers($uid);
    }

    // Get uid in existing user scenario.
    $uid = $user->id();

    // User exists, but not part of the team, so die.
    if (!empty($user) && checkTeam($uid) == FALSE) {
      header('HTTP/1.1 500 Email can\'t be added, please contact administrator | 5587');
      header('Content-Type: application/json; charset=UTF-8');
      die(json_encode(array('message' => 'ERROR', 'code' => 1337)));
    }

    // User exists and is part of the team, so add it.
    if (!empty($user) && checkTeam($uid) == TRUE) {
      // Add to channel.
      $channel->setUsers($uid);
      // Load user image
      $userImage = getUserImage($user);
      // Theme data into user_block_item single user in user_block.
      $toAppend = '<li>
                  <img src="' . $userImage . '" class="user-image-right" alt="User Image">
                  <a class="users-list-name" href="#">' . $user->getAccountName() . '</a>
                  <i class="fa fa-times remove-button">
                      <input type="hidden" name="uid" class="uid" value="' . $uid . '">
                      <input type="hidden" name="cid" class="cid" value="' . $cid . '">
                  </i>
              </li>';
    }

    // Append after the json call.
    $my_array = array(
      'cid' => $cid,
      'email' => $email,
      'toAppend' => $toAppend,
    );

    return new JsonResponse($my_array);

  }

  /**
   * Method to get the uid and names of users in channel.
   */
  public function getChannelUsers() {

    $cid = $_GET['cid'];
    $input = $_GET['input'];

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'getChannelUsers') != TRUE){ return FALSE; }

    // Query users in channel
    $query = \Drupal::database()->select('ol_user_channel', 'nfd');
    $query->addField('ufd', 'name', 'label');
    $query->addField('nfd', 'uid', 'id');
    $query->join('users_field_data', 'ufd', 'ufd.uid = nfd.uid');
    $query->condition('nfd.cid', $cid);
    $query->condition('ufd.name', $query->escapeLike($input) . '%', 'LIKE');
    $result = $query->execute()->fetchAll();

    return new JsonResponse($result);
  }

  /**
   * Function used by Ajax-call to edit the inline description without refresh.
   */
  public function alterDescription() {

    // Load parameter. Channel id.
    $description = $_GET['description'];
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'alterDescription') != TRUE){ return FALSE; }

    // Update channel description
    $query = \Drupal::database()->update('channel_field_data');
    $query->fields(['description' => $description]);
    $query->condition('id', $cid);
    $query->execute();

    $my_array = array(
      'cid' => $cid,
      'description' => $description,
    );
    return new JsonResponse($my_array);
  }

  /**
   * Function used by Ajax-call to edit the inline description without refresh.
   */
  public function alterName() {

    // Load parameter. Channel id.
    $channel_name = $_GET['channel_name'];
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'alterName') != TRUE){ return FALSE; }

    // Alter the name.
    $query = \Drupal::database()->update('channel_field_data');
    $query->fields(['name' => $channel_name]);
    $query->condition('id', $cid);
    $query->execute();

    $my_array = array(
      'cid' => $cid,
      'description' => $channel_name,
    );
    return new JsonResponse($my_array);
  }


  /**
   * Function used by Ajax-call to archive a channel
   */
  public function archiveChannel() {

    // Load parameter. Channel id.
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'archiveChannel') != TRUE){ return FALSE; }

    // Archive channel
    $query = \Drupal::database()->update('channel_field_data');
    $query->fields(['status' => 0]);
    $query->condition('id', $cid);
    $query->execute();

    $my_array = array(
      'cid' => $cid,
    );

    return new JsonResponse($my_array);
  }

  /**
   * Function used by Ajax-call to un-archive a channel
   */
  public function unarchiveChannel() {

    // Load parameter. Channel id.
    $cid = $_GET['cid'];
    $cid = \GuzzleHttp\json_decode($cid);

    // Check channel access, return false if check is false
    if(checkChannelAccess($cid, 'unarchiveChannel') != TRUE){ return FALSE; }

    // Unarchive the channel.
    $query = \Drupal::database()->update('channel_field_data');
    $query->fields(['status' => 1]);
    $query->condition('id', $cid);
    $query->execute();

    $my_array = array(
      'cid' => $cid,
    );

    return new JsonResponse($my_array);
  }

}
