<?php

namespace Drupal\openlucius_channel\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * File upload.
 */
class TrashController extends ControllerBase {

  /**
   * Returns a page with Trash can content.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getTrashcanContent() {

    global $base_path;

    // Get current user id
    $uid = \Drupal::currentUser()->id();

    // Get files in this channel.
    $query = \Drupal::database()->select('file_managed', 'fm');
    $query->fields('fm', ['filename', 'uri', 'filesize', 'created', 'filemime', 'fid', 'uid']);
    $query->addField('ufd','name');
    $query->addField('cfd','id', 'cid');
    $query->addField('cfd','type', 'channel_type');
    $query->join('ol_channel_files', 'ocf', 'ocf.fid = fm.fid');
    $query->join('users_field_data', 'ufd', 'ufd.uid = fm.uid');
    $query->join('channel_field_data', 'cfd', 'cfd.id = ocf.cid');
    $query->join('ol_user_channel', 'ouch', 'ouch.cid = ocf.cid');
    $query->condition('ouch.uid', $uid);
    $query->condition('fm.status', 2);
    $query->orderBy('fm.created', 'DESC');
    $files = $query->execute()->fetchAll();

    // Get detailed information for files.
    if (!empty($files)) {

      foreach ($files as $file) {

        // Get file data
        $name = $file->filename;
        $size = $file->filesize;
        $uri = $file->uri;
        $username = $file->name;
        $uid = $file->uid;
        $channel_id = $file->cid;
        $channel_name = get_channel_name($channel_id);
        $fid = $file->fid;
        $channel_type = $file->channel_type;
        $file_url = file_create_url($uri);
        $date = $file->created;
        $date =  gmdate("d M Y H:i", $date);
        $size = format_size($size);

        // Check if the file is an image. If yes thumbnail is image.
        if (strpos($file->filemime, 'image') !== false) {

          // Build thumbnail
          $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
          $img = $style->buildUrl($uri);

        } else {
          $img = $base_path . 'profiles/openlucius/themes/openluciustheme/assets/images/default-file.png';
        }

        // Add the items to array
        $files_data[] = array(
          'size' => $size,
          'name' => $name,
          'file_url' => $file_url,
          'fid' => $fid,
          'date' => $date,
          'img' => $img,
          'username' => $username,
          'uid' => $uid,
          'channel_name' => $channel_name,
          'channel_id' => $channel_id,
          'channel_type' => $channel_type,
          'path' => $base_path,
        );
      }
    }

    // Misc needed vars.
    $cacheTime = array('max-age' => 0);

    // Theme files in other template.
    $build = array(
      '#theme' => 'page_trash_can',
      '#files' => $files_data,
      '#cache' => $cacheTime,
    );

    return $build;

  }


  /**
   * Trashing files from right sidebar via AJAX
   *
   */
  public function trashFile() {

    // Load parameter: File id.
    $fid = $_GET['fid'];
    $fid = \GuzzleHttp\json_decode($fid);

    // Check file access, return false if check is false.
    if(checkFileAccess($fid, 'trashFile') != TRUE){ return FALSE; }

    // Trash file: set status to '2'
    $query = \Drupal::database()->update('file_managed');
    $query->fields(['status' => 2]);
    $query->condition('fid', $fid);
    $query->execute();

    $my_array = array(
      'fid' => $fid,
    );

    return new JsonResponse($my_array);
  }


  /**
   * Untrashing files from Trash can via AJAX
   *
   */
  public function untrashFile() {

    // Load parameter: Fileid.
    $fid = $_GET['fid'];
    $fid = \GuzzleHttp\json_decode($fid);

    // Check file access, return false if check is false.
    if(checkFileAccess($fid, 'untrashFile') != TRUE){ return FALSE; }

    // Untrash file: set status back to '1'
    $query = \Drupal::database()->update('file_managed');
    $query->fields(['status' => 1]);
    $query->condition('fid', $fid);
    $query->execute();

    $my_array = array(
      'fid' => $fid,
    );

    return new JsonResponse($my_array);
    }

}