<?php

namespace Drupal\openlucius_channel\Entity;

use Drupal\Core\Database\Database;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Channel entity.
 *
 * @ingroup openlucius_channel
 *
 * @ContentEntityType(
 *   id = "channel",
 *   label = @Translation("Channel"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\openlucius_channel\ChannelListBuilder",
 *     "views_data" = "Drupal\openlucius_channel\Entity\ChannelViewsData",
 *     "translation" = "Drupal\openlucius_channel\ChannelTranslationHandler",
 *
 *     "form" = {
 *       "default" = "Drupal\openlucius_channel\Form\ChannelForm",
 *       "add" = "Drupal\openlucius_channel\Form\ChannelForm",
 *       "edit" = "Drupal\openlucius_channel\Form\ChannelForm",
 *       "delete" = "Drupal\openlucius_channel\Form\ChannelDeleteForm",
 *     },
 *     "access" = "Drupal\openlucius_channel\ChannelAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\openlucius_channel\ChannelHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "channel",
 *   data_table = "channel_field_data",
 *   translatable = TRUE,
 *   admin_permission = "administer channel entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "name",
 *     "description" = "description",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *     "type" = "type",
 *     "closed" = "closed",
 *     "deleted" = "deleted",
 *   },
 *   links = {
 *     "canonical" = "/admin/structure/channel/{channel}",
 *     "add-form" = "/admin/structure/channel/add",
 *     "edit-form" = "/admin/structure/channel/{channel}/edit",
 *     "delete-form" = "/admin/structure/channel/{channel}/delete",
 *     "collection" = "/admin/structure/channel",
 *   },
 *   field_ui_base_route = "channel.settings"
 * )
 */
class Channel extends ContentEntityBase implements ChannelInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getId() {
    return $this->get('id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->get('description')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setDescription($description) {
    $this->set('description', $description);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setDeletedTime($timestamp) {
    $this->set('deleted', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getUsers() {

    // Retrieves a \Drupal\Core\Database\Connection which is a PDO instance.
    $connection = Database::getConnection();

    // Retrieves a PDOStatement object.
    $sth = $connection->select('ol_user_channel','x')
      ->fields('x')
      ->condition('cid', $this->id(), '=');

    // Execute the statement.
    $executed = $sth->execute();

    // Get all the results.
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);
    if (isset($results)) {
      return $results;
    }
    else {
      return array();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function checkSingleUser($uid) {

    // Retrieves a \Drupal\Core\Database\Connection which is a PDO instance
    $connection = Database::getConnection();

    // Retrieves a PDOStatement object
    $sth = $connection->select('ol_user_channel','x')
      ->fields('x')
      ->condition('cid', $this->id(), '=')
      ->condition('uid', $uid, '=');

    // Execute the statement
    $executed = $sth->execute();

    // Get all the results
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);
    if ($results[0]->uid == $uid){
      return TRUE;
    } else {
      return FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function setUsers($uid, $source) {

    // Check source, only throw this error when adding via right sidebar
    // Else there will be an error when people add a new channel
    if ($source == 'sidebar_right' && checkTeam($uid) == FALSE) {
      header('HTTP/1.1 500 user does not exist | 5473');
      header('Content-Type: application/json; charset=UTF-8');
      die(json_encode(array('message' => 'ERROR', 'code' => 1337)));
    }

    // For Add channel modal.
    if(checkTeam($uid) == FALSE){
      return;
    }

    // Check if record already exists
    $connection = Database::getConnection();
    $sth = $connection->select('ol_user_channel','x')
      ->fields('x')
      ->condition('cid', $this->id(), '=')
      ->condition('uid', $uid, '=');
    $executed = $sth->execute();
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);

    if (empty($results)){

      // Insert user in table.
      $sth = $connection->insert('ol_user_channel')
        ->fields(array('uid', 'cid'))
        ->values(array($uid, $this->id()));

      // Execute the statement
      $sth->execute();
    }
  }


  /**
   * {@inheritdoc}
   */
  public function removeUsers($uid) {

    // Retrieves a \Drupal\Core\Database\Connection which is a PDO instance
    $connection = Database::getConnection();

    // Check if record already exists
    $sth = $connection->select('ol_user_channel','x')
      ->fields('x')
      ->condition('cid', $this->id(), '=')
      ->condition('uid', $uid, '=');
    $executed = $sth->execute();
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);

    if (!$results == array()){

      // Remove user from table.
      $sth = $connection->delete('ol_user_channel')
        ->condition('cid', $this->id())
        ->condition('uid', $uid);

      // Execute the statement
      $sth->execute();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getMessages() {

    // Retrieves a \Drupal\Core\Database\Connection which is a PDO instance
    $connection = Database::getConnection();

// Retrieves a PDOStatement object
    $sth = $connection->select('message', 'x')
      ->fields('x')
      ->condition('cid', $this->id(), '=');

// Execute the statement
    $executed = $sth->execute();

// Get all the results
    $results = $executed->fetchAll(\PDO::FETCH_OBJ);
    if (isset($results)){
      return $results;
    } else {
      return '';
    }

  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Channel entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Name'))
      ->setDescription(t('The name of the Channel entity.'))
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    $fields['description'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Description'))
      ->setDescription(t('The description of the Channel entity.'))
      ->setSettings([
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -4,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);


    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the Channel is published.'))
      ->setDefaultValue(TRUE);

    $fields['type'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Channel type'))
      ->setDescription(t('An int indicating what type the channel is.'))
      ->setDefaultValue(TRUE);

    $fields['closed'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Closed or open channel status'))
      ->setDescription(t('A boolean indicating whether the Channel is open or closed.'))
      ->setDefaultValue(FALSE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['deleted'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Deleted'))
      ->setDescription(t('The time that the entity was Deleted.'))
      ->setDefaultValue(1);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));

    return $fields;
  }

}
