<?php

namespace Drupal\openlucius_channel\Entity;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityChangedInterface;
use Drupal\user\EntityOwnerInterface;

/**
 * Provides an interface for defining Channel entities.
 *
 * @ingroup openlucius_channel
 */
interface ChannelInterface extends  ContentEntityInterface, EntityChangedInterface, EntityOwnerInterface {

  // Add get/set methods for your configuration properties here.

  /**
   * Gets the Channel name.
   *
   * @return string
   *   Name of the Channel.
   */
  public function getName();

  /**
   * Sets the Channel name.
   *
   * @param string $name
   *   The Channel name.
   *
   * @return \Drupal\openlucius_channel\Entity\ChannelInterface
   *   The called Channel entity.
   */
  public function setName($name);

  /**
   * Gets the Channel creation timestamp.
   *
   * @return int
   *   Creation timestamp of the Channel.
   */
  public function getCreatedTime();

  /**
   * Sets the Channel creation timestamp.
   *
   * @param int $timestamp
   *   The Channel creation timestamp.
   *
   * @return \Drupal\openlucius_channel\Entity\ChannelInterface
   *   The called Channel entity.
   */
  public function setCreatedTime($timestamp);

  /**
   * Returns the Channel published status indicator.
   *
   * Unpublished Channel are only visible to restricted users.
   *
   * @return bool
   *   TRUE if the Channel is published.
   */
  public function isPublished();

  /**
   * Sets the published status of a Channel.
   *
   * @param bool $published
   *   TRUE to set this Channel to published, FALSE to set it to unpublished.
   *
   * @return \Drupal\openlucius_channel\Entity\ChannelInterface
   *   The called Channel entity.
   */
  public function setPublished($published);

}
