<?php
/**
 * @file
 * Contains \Drupal\openlucius_channel\Form\NewChannelForm.
 */

namespace Drupal\openlucius_channel\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Contribute form.
 */
class NewChannelForm extends FormBase {
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openlucius_channel_new_channel_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['name'] = array(
      '#type' => 'textfield',
      '#title' => t('Name'),
      '#placeholder' => t('e.g. sales'),
      '#required' => TRUE,
      '#attributes' => array(
        'class' => array('input-lg', 'form-control'),
        'maxlength' => array('24')
      ),
      '#prefix' => '<div class="modal-body"><div class="form-group">',
      '#suffix' => '</div>',
    );

    // Create a list of all usernames.
    $usersToSelect = array('--');
    foreach (User::loadMultiple() as $userToSelect) {
      array_push($usersToSelect, $userToSelect->getDisplayName());
    }

    $form['users'] = array(
      '#title' => t('Add existing users'),
      '#placeholder' => t('Search by name'),
      '#type' => 'textfield',
      '#autocomplete_route_name' => 'openlucius_channel.autocomplete',
      '#attributes' => array(
        'class' => array('input-lg', 'form-control'),
      ),
      '#suffix' => '<div class="form-group"></div>',
    );

    $form['emails'] = array(
      '#title' => t('Invite people by email'),
      '#placeholder' => t('john@example.com'),
      '#type' => 'textfield',
      '#required' => FALSE,
      '#attributes' => array(
        'class' => array('input-lg', 'form-control'),
      ),
      '#suffix' => '<div class="form-group"></div>',
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Create Channel'),
      '#attributes' => array('class' => array('btn', 'btn-success')),
      '#prefix' => '</div><div class="modal-footer">',
    );

    $form['button'] = array(
      '#type' => 'button',
      '#value' => t('Cancel'),
      '#attributes' => array(
        'class' => array('btn', 'btn-default', 'pull-left'),
        'data-dismiss' => array('modal')
      ),
      '#suffix' => '</div>',
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    global $base_path;

    // Fill array with form values.
    $formContent = $form_state->getValues();

    // Create new channel.
    $channel = \Drupal::entityTypeManager()
      ->getStorage('channel')
      ->create(array('name' => $formContent['name'], 'description' => ' '));
    $channel->save();

    // Load newly created channel.
    $newChannelId = $channel->getId();

    // Read email field, build array of email values.
    $emails = preg_replace('/\s+/', '', $formContent['emails']);
    $emails = explode(",", $emails);

    // Loop through form submission
    foreach ($emails as $email) {

      // Check if email is valid.
      if (\Drupal::service('email.validator')->isValid($email)) {

        // User doesn't exist, create it.
        if (!user_load_by_mail($email)) {

          // Create new user.
          $user = \Drupal\user\Entity\User::create();
          $user->enforceIsNew();
          $user->setEmail($email);
          $user->setUsername($email);
          $user->activate();
          $user->save();
          _user_mail_notify('register_no_approval_required', $user);
          $user = user_load_by_mail($email);
          $uid = $user->id();
          addToTeam($uid);
          $channel->setUsers($uid);
        }

        // User does exist.
        else {

          // Get user data.
          $user = user_load_by_mail($email);
          $uid = $user->id();

          // User in team, so add it.
          if (checkTeam($uid) == TRUE) {
            $channel->setUsers($uid);
          }
        }
      }
    }

    // Add creator to channel.
    $uid = \Drupal::currentUser()->id();
    $channel->setUsers($uid);

    // Add given existing users to channel.
    if (isset($formContent['users'])) {

      // Remove all spaces after comma's, else username won't match
      $pattern = '/\s*,\s*/';
      $replace = ',';
      $userNames = preg_replace($pattern, $replace, $formContent['users']);

      // Loop through all user, verify and add to channel
      $userNames = explode(',', $userNames);
      foreach ($userNames as $userName) {
        if ($userName != '') {
          $user = user_load_by_name($userName);
          if ($user) {
            $channel->setUsers(intval($user->id()));
          }
        }
      }
    }
    // Redirect user to newly created channel.
    $response = new RedirectResponse($base_path.'channel/'.$newChannelId.'/message');
    $response->send();
    return;

  }

}
