<?php
/**
 * @file
 * Contains \Drupal\openlucius_channel\Form\NewDMForm.
 */

namespace Drupal\openlucius_channel\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Contribute form.
 */
class NewDMForm extends FormBase {
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openlucius_channel_new_dm_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['#attributes']['onsubmit'] = 'return validateForm()';

    $form['users'] = array(
      '#placeholder' => t('Search by name'),
      '#type' => 'textfield',
      '#autocomplete_route_name' => 'openlucius_channel.autocomplete',
      '#attributes' => array(
        'class' => array('input-lg', 'form-control'),
      ),
      '#prefix' => '<div class="modal-body"><div class="form-group">',
      '#suffix' => '</div>',
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Create private message'),
      '#attributes' => array('class' => array('btn', 'btn-success')),
      '#prefix' => '</div><div class="modal-footer">',
    );
    $form['button'] = array(
      '#type' => 'button',
      '#value' => t('Cancel'),
      '#attributes' => array(
        'class' => array('btn', 'btn-default', 'pull-left'),
        'data-dismiss' => array('modal')
      ),
      '#suffix' => '</div>',
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    global $base_path;

    // Fill array with form values.
    $formContent = $form_state->getValues();

    // Remove all spaces after comma's, else username won't match.
    $pattern = '/\s*,\s*/';
    $replace = ',';
    $userNames = preg_replace($pattern, $replace, $formContent['users']);
    $extraUserNames = explode(",", $userNames);

    // Set needed stuff.
    $currentUserName = \Drupal::currentUser()->getAccountName();
    $channelNameString = $currentUserName . ' and ' . $extraUserNames;

    // Create new channel.
    $channel = \Drupal::entityTypeManager()
      ->getStorage('channel')
      ->create(array('name' => $channelNameString, 'type' => '2', 'description' => ' '));
    $channel->save();

    // Loop through submission and add users to direct message
    foreach ($extraUserNames as $extraUserName) {

      $extraUser = user_load_by_name($extraUserName);
      $extraUserId = $extraUser->id();

      // Add given users to channel if it is team member.
      if (checkTeam($extraUserId) == TRUE) {
        $channel->setUsers($extraUserId);
      }
    }

    // Add creator to channel.
    $uid = \Drupal::currentUser()->id();
    $channel->setUsers($uid);

    // Redirect user to newly created channel.
    $newChannelId = $channel->getId();
    $response = new RedirectResponse($base_path .'channel/' . $newChannelId . '/message');
    $response->send();

  }
}
