<?php
/**
 * @file
 * Contains \Drupal\openlucius_channel\Form\NewTaskForm.
 */

namespace Drupal\openlucius_channel\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\openlucius_channel\Entity\Channel;

/**
 * Contribute form.
 */
class NewTaskForm extends FormBase {
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openlucius_channel_task_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form['#attributes']['onsubmit'] = 'return validateForm()';

    $form['names'] = array(
      '#type' => 'textarea',
      '#title' => t('1 task per line'),
      '#placeholder' => t('e.g. "do this stuff"'),
      '#required' => TRUE,
      '#attributes' => array(
        'class' => array('input-lg', 'form-control'),
        'maxlength' => array('4028')
      ),
      '#prefix' => '<div class="modal-body"><div class="form-group">',
      '#suffix' => '</div>',
    );

    $form['cid'] = array(
      '#type' => 'hidden',
      '#attributes' => array('id' => array('cid')),
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Create'),
      '#attributes' => array('class' => array('btn', 'btn-success')),
      '#prefix' => '</div><div class="modal-footer">',
    );
    $form['button'] = array(
      '#type' => 'button',
      '#value' => t('Cancel'),
      '#attributes' => array(
        'class' => array('btn', 'btn-default', 'pull-left'),
        'data-dismiss' => array('modal')
      ),
      '#suffix' => '</div>',
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    global $base_path;

    // Get form content
    $form_content = $form_state->getValues();
    $task_data= $form_content['names'];
    $parent_channel = $form_content['cid'];
    $uid = \Drupal::currentUser()->id();

    $task_names = preg_split('/\r\n|[\r\n]/', $task_data);

    foreach($task_names as $task_name){

      // Skip empty lines
      if(empty($task_name)){
        continue;
      }

      // Create new task (channel).
      $channel = \Drupal::entityTypeManager()
        ->getStorage('channel')
        ->create(array('name' => $task_name, 'type' => '3', 'description' => ' '));
      $channel->save();

      // Add creator to channel.
      $channel->setUsers($uid);
      $newChannelId = $channel->getId();

      // Get users from parent channel.
      $parentChannelObject = Channel::load($parent_channel);
      $ParentChannelUsers = $parentChannelObject->getUsers();

      // Add users in 'Task channel'.
      foreach ($ParentChannelUsers as $user) {
        $channel->setUsers($user->uid);
      };

      // Insert task data
      $query = \Drupal::database()->insert('ol_tasks');
      $query->fields([
        'tid',
        'parent_id',
        'assigned',
      ]);
      $query->values([
        $newChannelId,
        $parent_channel,
        0,
      ]);
      $query->execute();
    }

    // If only 1 task is added, redirect to new task.
    if(count($task_names) == 1){
      $redirect_url = $newChannelId;
    }
    // Else, redirect to parent channel.
    else{
      $redirect_url = $parent_channel;
    }

    drupal_set_message(t('Created! You can find it in the left sidebar.'), 'status');

    // Redirect user to parent channel.
    $response = new RedirectResponse($base_path .'channel/' . $redirect_url . '/message');
    $response->send();

  }
}
