<?php

namespace Drupal\openlucius_config\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Form Configuration.
 *
 * @package Drupal\openlucius_config\Form
 */
class OpenluciusFormConfiguration extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return array(
      'openlucius_config.settings',
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openlucius_config_form';
  }

  /**
   * Get all available settings.
   */
  public function getAvailableSettings() {
    return array(
      'openlucius_nodejs_ip',
      'openlucius_nodejs_port',
      'openlucius_intro_message',

    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('openlucius_config.settings');

    // The NodeJS configuration.
    $form['nodejs'] = array(
      '#type' => 'details',
      '#title' => t('Chat server settings'),
      '#group' => 'advanced',
    );
    $form['nodejs']['openlucius_nodejs_ip'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Ip address of the chat server'),
      '#required' => TRUE,
      '#default_value' => $config->get('openlucius_nodejs_ip'),
    );
    $form['nodejs']['openlucius_nodejs_port'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Port number of the chat server'),
      '#required' => TRUE,
      '#default_value' => $config->get('openlucius_nodejs_port'),
    );

    // The homepage configuration.
    $form['homepage'] = array(
      '#type' => 'details',
      '#title' => t('Homepage settings'),
      '#group' => 'advanced',
    );
    $form['homepage']['openlucius_intro_message'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Message to display on the homepage'),
      '#required' => TRUE,
      '#default_value' => $config->get('openlucius_intro_message'),
    );
    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = \Drupal::service('config.factory')
      ->getEditable('openlucius_config.settings');

    $available_settings = $this->getAvailableSettings();

    if (!empty($available_settings)) {
      foreach ($available_settings as $setting) {
        $config->set($setting, $form_state->getValue($setting));
      }
    }
    $config->save();

    parent::submitForm($form, $form_state);
  }
}