<?php

namespace Drupal\openlucius_core\EventSubscriber;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;


/**
 * Event subscriber subscribing to KernelEvents::REQUEST.
 */
class RedirectAnonymousSubscriber implements EventSubscriberInterface {

  /**
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   */
  public function checkAuthStatus(GetResponseEvent $event) {

    global $base_url;

    $route_name = \Drupal::routeMatch()->getRouteName();
//    file_put_contents('test.txt', $route_name);
    if (
      \Drupal::currentUser()->isAnonymous() &&
      \Drupal::routeMatch()->getRouteName() != 'user.login' &&
      \Drupal::routeMatch()->getRouteName() != 'user.login.http' &&
      \Drupal::routeMatch()->getRouteName() != 'user.reset' &&
      \Drupal::routeMatch()->getRouteName() != 'user.reset.form' &&
      \Drupal::routeMatch()->getRouteName() != 'user.reset.login' &&
      \Drupal::routeMatch()->getRouteName() != 'system.csrftoken' &&
      \Drupal::routeMatch()->getRouteName() != 'rest.get_channels.GET.json' &&
      \Drupal::routeMatch()->getRouteName() != 'user.pass'
    ) {

      // Add logic to check other routes you want available to anonymous users.
      // Otherwise, redirect to login page.
      if (strpos($route_name, 'view') === 0 && strpos($route_name, 'rest_') !== FALSE) {
        return;
      }

      $response = new RedirectResponse($base_url .'/user/login', 301);
      $event->setResponse($response);
      $event->stopPropagation();
      return;
    }

    // Check to make sure user will not manually go to the user profile page.
    if (!\Drupal::currentUser()->isAnonymous() &&
      \Drupal::routeMatch()->getRouteName() == 'user.page'
    ) {
      $response = new RedirectResponse($base_url .'home', 301);
      $event->setResponse($response);
      $event->stopPropagation();

      return;
    }

  }

  /**
   * @return mixed
   */
  public static function getSubscribedEvents() {
    $events[KernelEvents::REQUEST][] = array('checkAuthStatus');
    return $events;
  }

}
