<?php

namespace Drupal\openlucius_core\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Url;
use Drupal\file\Entity\File;
use Drupal\user\Entity\User;


/**
 * Provides a 'ChannelListBlock' block.
 *
 * @Block(
 *  id = "channel_list_block",
 *  admin_label = @Translation("Channel list block"),
 * )
 */
class ChannelListBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {

    global $base_path;

    // Check for frontpage, for correct switching of left menu.
    if (\Drupal::service('path.matcher')->isFrontPage()) {
      $frontpage = 1;
    }
    else {
      if (\Drupal::routeMatch()->getRouteName() == 'user.reset.form') {
        $frontpage = 3;
      }
      else {
        $frontpage = 2;
      }
    }

    // Get current user id
    $uid = \Drupal::currentUser()->id();
    $username = \Drupal::currentUser()->getAccountName();

    // Get current channel id
    $current_cid = \Drupal::request()->get('channelid');

    // Get modal form data for new Channel
    $form = \Drupal::formBuilder()
      ->getForm('Drupal\openlucius_channel\Form\NewChannelForm');

    // Get modal form data for new Private message
    $formdm = \Drupal::formBuilder()
      ->getForm('Drupal\openlucius_channel\Form\NewDMForm');

    // Get modal form data for new Task
    $formTask= \Drupal::formBuilder()
      ->getForm('Drupal\openlucius_channel\Form\NewTaskForm');

    // Get Channels data
    $query = \Drupal::database()->select('ol_user_channel', 'ouc');
    $query->fields('ouc', ['cid']);
    $query->addField('cfd', 'name');
    $query->addField('cfd', 'type');
    $query->join('channel_field_data', 'cfd', 'ouc.cid = cfd.id');
    $query->condition('ouc.uid', $uid, '=');
    $query->condition('cfd.status', 1);
    $query->orderBy('cfd.name', 'ASC');
    $channels = $query->execute()->fetchAll();

    // Initialize arrays
    $channeldata = array();
    $dmdata = array();

    // Check if there are results
    if (isset($channels)) {

      // Build channel data for frontend
      foreach ($channels as $channel) {

        // Check for tasks in channel
        $query = \Drupal::database()->select('channel_field_data', 'cfd');
        $query->addField('cfd', 'name');
        $query->addField('cfd', 'id');
        //$query->addField('olt','parent_id');
        $query->addField('ufd','name','assignee');
        //$query->addField('fma','filename');
        $query->Join('ol_tasks', 'olt', 'olt.tid = cfd.id');
        $query->Join('users_field_data', 'ufd', 'ufd.uid = olt.assigned');
        //$query->join('user__field_user_image', 'ufui', 'ufui.entity_id = olt.assigned');
        //$query->join('file_managed', 'fma', 'fma.fid = ufui.field_user_image_target_id');
        $query->condition('olt.parent_id', $channel->cid);
        $query->condition('cfd.status', 1);
        $query->orderBy('cfd.name', 'ASC');
        $tasks = $query->execute()->fetchAll();


        // Build array for Channels with tasks in left sidebar
        if ($channel->type == '1') {
          $channeldata[] = array(
            'cid'   => $channel->cid,
            'name' => $channel->name,
            'tasks' => $tasks,
          );
        }

        // Build array for Private Messages in left sidebar
        if ($channel->type == '2') {

          $display_names = array();

          // Query names of 'others' in this private message
          $query = \Drupal::database()->select('ol_user_channel', 'ouc');
          $query->addField('ufd', 'name');
          $query->join('users_field_data', 'ufd', 'ouc.uid = ufd.uid');
          $query->condition('ouc.uid', $uid, '!=');
          $query->condition('ouc.cid', $channel->cid, '=');
          $query->orderBy('ufd.name', 'ASC');
          $names = $query->execute()->fetchAll();

          foreach ($names as $name) {
            $display_names[] = $name;
          }

          // Exception for 'user own' private channel
          if(empty($display_names)){
            $display_names[]->name = $username;
          }

          $dmdata[] = array(
            'cid' => $channel->cid,
            'name' => $display_names,
            'tasks' => $tasks,
          );
        }
      }
    }

    // Misc needed vars
    $cacheTime = array('max-age' => 0);
    $site_name = \Drupal::config('system.site')->get('name');

    // Initialize the menu dropdowns array.
    $menu_dropdowns = [];

    // Generate the links.
    $profile_link = Url::fromRoute('openlucius_usermanager.profile', ['profileId' => $uid]);
    $all_team_members_link = Url::fromRoute('openlucius_usermanager.users');
    $archived_channels_link = Url::fromRoute('openlucius_channel.page_archived_channel');
    $trash_can_link = Url::fromRoute('openlucius_channel.trash_can');

    // Add the links to the array.
    $menu_dropdowns[$profile_link->toString()] = $this->t('My profile');
    $menu_dropdowns[$all_team_members_link->toString()] = $this->t('All Team Members');
    $menu_dropdowns[$archived_channels_link->toString()] = $this->t('Archive');
    $menu_dropdowns[$trash_can_link->toString()] = $this->t('Trash Can');

    // Allow other modules to override the array.
    \Drupal::moduleHandler()->alter('openlucius_user_dropdown_links', $menu_dropdowns);

    // Add them to the build.
    $build = array(
      '#theme' => 'channel_list_block',
      '#items' => $channeldata,
      '#dmsg' => $dmdata,
      '#form' => $form,
      '#formdm' => $formdm,
      '#formtask' => $formTask,
      '#base_path' => $base_path,
      '#user' => $username,
      '#current_cid' => $current_cid,
      '#uid' => $uid,
      '#frontpage' => $frontpage,
      '#site_name' => $site_name,
      '#cache' => $cacheTime,
      '#menu_dropdowns' => $menu_dropdowns,
    );

    // TODO: cache optimization
    /*
    '#cache'=>[
        'contexts'=> [
          'user',
        ],
        'tags' => [
          'block:channel_list',
        ],
      ],
    */

    return $build;
  }

}
