<?php

namespace Drupal\openlucius_core\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\openlucius_channel\Entity\Channel;

/**
 * Provides a 'ChannelTitleBlock' block.
 *
 * @Block(
 *  id = "channel_title_block",
 *  admin_label = @Translation("Channel title block"),
 * )
 */
class ChannelTitleBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {

    global $base_path;

    // Get channel id.
    $cid = \Drupal::request()->attributes->get('channelid');

    // Get needed Channel users for count.
    $query = \Drupal::database()->select('ol_user_channel', 'x');
    $query->fields('x');
    $query->condition('cid', $cid);
    $channel_users = $query->execute()->fetchAll();

    // Count the users.
    $userCount = count($channel_users);


    // Check if there is a cid, if not: we are not in a channel, choose alternative
    if (empty($cid) ){

      // Get page title
      $request = \Drupal::request();
      if ($route = $request->attributes->get(\Symfony\Cmf\Component\Routing\RouteObjectInterface::ROUTE_OBJECT)) {
        $title = \Drupal::service('title_resolver')->getTitle($request, $route);
      }

      // No channel page
      $cacheTime = array('max-age' => 0);
      $build = array(
        '#theme' => 'channel_title_block',
        '#channel_status' => 2,
        '#names' => $title,
        '#cache' => $cacheTime,
      );

      return $build;
    }

    // Get needed vars.
    $uid = \Drupal::currentUser()->id();
    $username = \Drupal::currentUser()->getAccountName();

    // Get needed Channel data.
    $query = \Drupal::database()->select('channel_field_data', 'cfd');
    $query->fields('cfd', ['status','description','name','type','id']);
    $query->condition('cfd.id', $cid);
    $query->range(0, 1);
    $channel_data = $query->execute()->fetchAssoc();

    $display_names = array();

    // Check if this is a Private message, if so: alter stuff
    if ($channel_data['type'] == '2') {

      // Query names of 'others' in this private message
      $query = \Drupal::database()->select('ol_user_channel', 'ouc');
      $query->addField('ufd', 'name');
      $query->join('users_field_data', 'ufd', 'ouc.uid = ufd.uid');
      $query->condition('ouc.uid', $uid, '!=');
      $query->condition('ouc.cid', $cid, '=');
      $query->orderBy('ufd.name', 'ASC');
      $names = $query->execute()->fetchAll();

      // Build array with user names.
      foreach ($names as $name) {
        $display_names[] = $name;
      }
      // Exception for 'user own' private channel
      if(empty($display_names)){
        $display_names[]->name = $username;
      }
    }

    // No Private message, plain Channel.
    else{
      $display_names[]->name = $channel_status = $channel_data['name'];;
    }

    // Set cache to 0 for now.
    $cacheTime = array('max-age' => 0);
    $base_path = $base_path;

    // Add them to the build.
    $build = array(
      '#theme' => 'channel_title_block',
      '#channel_type' => $channel_data['type'],
      '#channel_id' => $channel_data['id'],
      '#names' => $display_names,
      '#description' => $channel_data['description'],
      '#channel_status' => $channel_data['status'],
      '#base_path' => $base_path,
      '#user_count' => $userCount,
      '#cache' => $cacheTime,
    );

    return $build;
  }

}
