<?php

namespace Drupal\openlucius_core\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\file\Entity\File;
use Drupal\openlucius_channel\Entity\Channel;
use Drupal\user\Entity\User;

/**
 * Provides a 'UserListBlock' block.
 *
 * @Block(
 *  id = "user_list_block",
 *  admin_label = @Translation("User list block"),
 * )
 */
class UserListBlock extends BlockBase {

  /**
   * {@inheritdoc}
   */
  public function build() {

    global $base_path;

    // Get channel id.
    $cid = \Drupal::request()->attributes->get('channelid');

    $form = \Drupal::formBuilder()
      ->getForm('Drupal\openlucius_channel\Form\AddUserForm');

    $form_remove = \Drupal::formBuilder()
      ->getForm('Drupal\openlucius_channel\Form\RemoveUserForm');


    // Get creator name and date created info
    $query = \Drupal::database()->select('channel_field_data', 'cfd');
    $query->fields('cfd',['created','name','description','type']);
    $query->addField('ufd','name');
    $query->join('users_field_data', 'ufd', 'ufd.uid = cfd.user_id');
    $query->condition('cfd.id', $cid, '=');
    $channel_info = $query->execute()->fetchAll();

    // Turn queried data into twig ready stuff
    $channel_name = $channel_info[0]->name;
    $creator_name = $channel_info[0]->ufd_name;
    $channel_description = $channel_info[0]->description;
    $created_date = gmdate("Y-m-d H:i", $channel_info[0]->created);
    $channel_type = $channel_info[0]->type;

    // Get files in this channel.
    $query = \Drupal::database()->select('file_managed', 'fm');
    $query->fields('fm', ['filename', 'uri', 'filesize', 'created', 'filemime', 'fid','uid']);
    $query->addField('ufd','name');
    $query->join('ol_channel_files', 'ocf', 'ocf.fid = fm.fid');
    $query->join('users_field_data', 'ufd', 'ufd.uid = fm.uid');
    $query->condition('ocf.cid', $cid);
    $query->condition('fm.status', 1);
    $query->orderBy('fm.created', 'DESC');
    $files = $query->execute()->fetchAll();

    // Get detailed information for files.
    if (!empty($files)) {

      foreach ($files as $file) {
        $name = $file->filename;
        $size = $file->filesize;
        $uri = $file->uri;
        $username = $file->name;
        $fid = $file->fid;
        $uid = $file->uid;

        $file_url = file_create_url($uri);
        $date = $file->created;

        // Check if the file is an image. If yes thumbnail is image.
        if (strpos($file->filemime, 'image') !== false) {

          // Build thumbnail
          $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
          $img = $style->buildUrl($uri);

          // $img = file_create_url($file->uri);

        } else {
          $img = $base_path . 'profiles/openlucius/themes/openluciustheme/assets/images/default-file.png';
        }
        $date =  gmdate("d M Y H:i", $date);
        $size = format_size($size);

        // Add the items to array
        $files_array[] = array(
          'size' => $size,
          'name' => $name,
          'file_url' => $file_url,
          'fid' => $fid,
          'date' => $date,
          'img' => $img,
          'username' => $username,
          'uid' => $uid,
        );
      }
      $files_count = count($files_array);
    }
    // Get userdata.
    $getUsers = NULL;

    if (!empty($cid)) {
      $channel = Channel::load($cid);
      $getUsers = $channel->getUsers();
    }

    // Count the users.
    $userCount = count($getUsers);

    // Check if there are users.
    if (!empty($getUsers)) {

      // Loop through the users.
      foreach ($getUsers as $uid) {
        $uid = $uid->uid;

        if (User::load($uid) != NULL) {

          // Load user.
          $user = User::load($uid);

          // Load the name of the user.
          $name = $user->getAccountName();

          // Get user image.
          $userImage = getUserImage($user);

          $profile_link = $base_path .'users/profile/' .$uid;

          // Add the items to array.
          $users_array[] = array(
            'uid' => $uid,
            'name' => $name,
            'img' => $userImage,
            'profile_link' => $profile_link,
          );
        }
      }
    }

    // Get data for Task channel
    if($channel_type == 3) {
      $query = \Drupal::database()->select('ol_tasks', 'olt');
      $query->addField('olt', 'assigned');
      $query->addField('olt', 'duedate');
      $query->addField('olt', 'parent_id');
/*      $query->addField('cfd', 'id');
      $query->join('channel_field_data', 'cfd', 'cfd.id = olt.parent_id');*/
      $query->condition('olt.tid', $cid, '=');
      $task_data = $query->execute()->fetchAssoc();

      $parentChannelNameArray = get_channel_name($task_data['parent_id']);
      $parentChannelName = $parentChannelNameArray[0]->name;

    }

    // Set the max age.
    $cacheTime = array('max-age' => 0);
    if(empty($files_count)){
      $files_count = '0';
    }

    // Theme files in other template.
    $files = array(
      '#theme' => 'file_block',
      '#files' => $files_array,
      '#path' => $base_path,
      '#cache' => $cacheTime,
      '#cid' => $cid,
    );

    // Theme users in other template.
    $users = array(
      '#theme' => 'user_block',
      '#users' => $users_array,
      '#channel_type' => $channel_type,
      '#cache' => $cacheTime,
      '#cid' => $cid,
    );

    // Theme assigned to
    $user_assigned_to_list = array(
      '#theme' => 'user_assigned_to_list',
      '#users_in_channel' => $users_array,
      '#assigned_uid' => $task_data['assigned'],
      '#cache' => $cacheTime,
    );

    // Theme due date
    $due_date_block = array(
      '#theme' => 'task_due_date',
      '#due_date' => $task_data['duedate'],
      '#cache' => $cacheTime,
    );

    // Add them to the build.
    $build = array(
      '#theme' => 'user_list_block',
      '#user_count' => $userCount,
      '#users' => $users,
      '#files' => $files,
      '#form' => $form,
      '#form_remove' => $form_remove,
      '#cid' => $cid,
      '#creator_name' => $creator_name,
      '#created_date' => $created_date,
      '#channel_name' => $channel_name,
      '#channel_description' => $channel_description,
      '#channel_type' => $channel_type,
      '#files_count' => $files_count,
      '#user_assigned_to_list' => $user_assigned_to_list,
      '#due_date_block' => $due_date_block,
      '#base_path' => $base_path,
      '#parent_channel_name' => $parentChannelName,
      '#parent_channel_id' => $task_data['parent_id'],
      '#cache' => $cacheTime,
    );

    return $build;
  }
}
