<?php

namespace Drupal\openlucius_usermanager\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Provides route responses for the Example module.
 */
class ProfileController extends ControllerBase {

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getPageContent($profileId) {

    global $base_path;

    // Get user basic profile data
    $uid = \Drupal::currentUser()->id();

    // Get all shared channels
    $query = \Drupal::database()->select('ol_user_channel', 'ouc');
    $query->fields('ouc', ['cid']);
    $query->addField('oouc', ['uic']);
    $query->join('ol_user_channel', 'oouc', 'oouc.cid = ouc.cid AND oouc.uid=' . $profileId);
    $query->condition('ouc.uid', $uid);
    $cid = $query->execute()->fetchAllAssoc('cid');

    // No shared channels, no access.
    if (empty($cid)) {
      $response = new RedirectResponse($base_path . "home");
      $response->send();
    }

    // So we have shared channels, get all file data in there.
    foreach ($cid as $id) {
      $query = \Drupal::database()->select('file_managed', 'fm');
      $query->fields('fm', [
        'filename',
        'uri',
        'filesize',
        'created',
        'filemime',
      ]);
      $query->addField('ufd', 'name');
      $query->addField('cfd', 'name');
      $query->join('ol_channel_files', 'ocf', 'ocf.fid = fm.fid');
      $query->join('users_field_data', 'ufd', 'ufd.uid = fm.uid');
      $query->join('channel_field_data', 'cfd', 'cfd.id = ocf.cid');
      $query->condition('ufd.uid', $profileId, '=');
      $query->condition('ocf.cid', intval($id->cid), '=');
      $query->condition('fm.status', 1);
      $query->orderBy('fm.created', 'DESC');
      $files = $query->execute()->fetchAll();

      foreach ($files as $file) {

        if (strpos($file->filemime, 'image') !== false) {

          // Build thumbnail
          $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
          $img = $style->buildUrl($file->uri);

        }
        else {
          $img = $base_path . 'profiles/openlucius/themes/openluciustheme/assets/images/default-file.png';
        }

        $channel_name = get_channel_name($id->cid);

        // Create channel link
        //$channel_link = '<a href="' .$base_path . 'channel/'.$id->cid .'/message ">'.$file->cfd_name .'</a>';
        $files_array[] = array(
          'size' => format_size($file->filesize),
          'name' => $file->filename,
          'file_url' => file_create_url($file->uri),
          'date' => gmdate("d M Y H:i", $file->created),
          'img' => $img,
          'username' => $file->name,
          'channel_name' => $channel_name,
          'channel_id' => $id->cid,
          'path' => $base_path,
        );
      }

      // Get all assigned tasks in shared channel
      $query = \Drupal::database()->select('ol_tasks', 'olt');
      $query->addField('olt', 'tid');
      $query->addField('olt', 'duedate');
      $query->addField('cfd', 'name');
      $query->join('channel_field_data', 'cfd', 'cfd.id = olt.tid');
      $query->condition('olt.assigned', $profileId);
      $query->condition('cfd.id', intval($id->cid), '=');
      $query->condition('cfd.status', 1);
      $tasks_data = $query->execute()->fetchAll();

      // Convert task data for twigging
      foreach ($tasks_data as $task) {

        // Get parent channel

        // Get all assigned tasks data
        $query = \Drupal::database()->select('channel_field_data', 'cfd');
        $query->addField('cfd', 'id');
        $query->addField('cfd', 'name');
        $query->join('ol_tasks', 'olt', 'olt.parent_id = cfd.id');
        $query->condition('olt.tid', $task->tid);
        $query->condition('cfd.status', 1);
        $parent_data = $query->execute()->fetchAssoc();


        // Add the items to array
        $tasks_array[] = array(
          'tid' => $task->tid,
          'name' => $task->name,
          'duedate' => $task->duedate,
          'parent_id' => $parent_data['id'],
          'parent_name' => $parent_data['name'],
        );
      }
    }

    // Sort the data with duedate ascending
    foreach ($tasks_array as $key => $row) {
      $duedate[$key]  = $row['duedate'];
    }
    array_multisort($duedate, SORT_ASC, $tasks_array);

    // Get user info.
    $user = \Drupal\user\Entity\User::load($profileId);
    $name = $user->getUsername();
    $email = $user->getEmail();

    if (!$user->field_user_image->isEmpty()) {

      // Build user picture with image style
      $picture_url = $user->get('field_user_image')->entity->getFileUri();
      $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
      $picture = $style->buildUrl($picture_url);

    }
    else {
      $picture = $base_path . 'profiles/openlucius/themes/openluciustheme/assets/images/default-user.png';
    }

    // Produce 'edit profile' link
    if($profileId == $uid) {
      $profile_edit_link = $base_path . 'user/' . $profileId . '/edit/';
    }

    // Add the items to array.
    $data = array(
      'uid' => $profileId,
      'userimage' => $picture,
      'name' => $name,
      'email' => $email,
      'profile_edit_link' => $profile_edit_link,
    );

    // Set the max age.
    $cacheTime = array('max-age' => 0);

    $element = array(
      '#theme' => 'profile_block',
      '#data' => $data,
      '#cache' => $cacheTime,
      '#total_files' => $files_array,
      '#tasks_data' => $tasks_array,
      '#path' => $base_path,
    );
    return $element;
  }
}