<?php

namespace Drupal\openlucius_usermanager\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\file\Entity\File;
use Drupal\user\Entity\User;

/**
 * Provides route responses for the Example module.
 */
class UsersController extends ControllerBase {

  /**
   * Returns a simple page.
   *
   * @return array
   *   A simple renderable array.
   */
  public function getPageContent() {
    $data = array();

    // Get the user's team id.
    $query = \Drupal::database()->select('ol_user_team', 'ost');
    $query->addField('ost', 'team_id');
    $query->condition('ost.uid', \Drupal::currentUser()->id());
    $query->range(0, 1);
    $teamId = $query->execute()->fetchField();

    // Get all the users that belong to the same team.
    $query = \Drupal::database()->select('ol_user_team', 'ost');
    $query->addField('ost', 'uid');
    $query->condition('ost.team_id', $teamId);
    $getUsers = $query->execute()->fetchAll();

    $removeButtons = FALSE;
    $user = User::load(\Drupal::currentUser()->id());
    if ($user->hasRole('moderator')) {
      // Enable remove user buttons.
      $removeButtons = TRUE;
    }

    // Check if there are users.
    if (!empty($getUsers)) {

      // Loop through the uids.
      foreach ($getUsers as $item) {
        // Load user.
        $user = User::load($item->uid);

        // Load the name of the user.
        $name = $user->getAccountName();

        $user_email = $user->getEmail();

        // Get user image.
        $userImage = getUserImage($user);

        // Build thumbnail
/*        $style = \Drupal::entityTypeManager()->getStorage('image_style')->load('thumbnail');
        $userImage = $style->buildUrl($image_uri);*/


        // Add the items to array.
        $data[] = array(
          'uid' => $item->uid,
          'name' => $name,
          'user_email' => $user_email,
          'img' => $userImage,
        );
      }
    }

    // Set the max age.
    $cacheTime = array('max-age' => 0);

    $element = array(
      '#theme' => 'usermanager_block',
      '#data' => $data,
      '#removebuttons' => $removeButtons,
      '#cache' => $cacheTime,
    );
    return $element;
  }

  /**
   * Ajax function to remove users from a team.
   *
   * @return JsonResponse
   *   A json to be read by frontend.
   */
  public function delete() {

    // Load parameter. Channel id.
    $uid = $_GET['uid'];
    $uid = \GuzzleHttp\json_decode($uid);

    // Delete users from channels
    // TODO: we don't know wich channels belong to wich team, so we can't delete a user from 'all channels in my team'
    // So we probably need to add an extra database table/row somewhere
    // For now: just remove from 'my team' and manually kick from all channels to 'block a user'
    // !) Don't block, because multiple teams will apply in future.

     /*
      $query = \Drupal::database()->select('ol_user_channel', 'c');
      $query->addField('c', 'cid');
      $query->addField('t', 'team_id');
      $query->join('ol_user_team', 't', 'c.uid = t.uid');
      $query->condition('c.uid', $uid);
      $query->condition('t.team_id', getTeamId());
      $result = $query->execute()->fetchAll();

      \Drupal::logger('remove user')->notice(print_r($result));
    */

    // Delete users from teams
    $query = \Drupal::database()->delete('ol_user_team');
    $query->condition('uid', $uid);
    $query->condition('team_id', getTeamId());
    $query->execute();

    $my_array = array(
      'uid' => $uid,
    );

    return new JsonResponse($my_array);
  }
}
