/**
 * @file
 * Checks for internet connection.
 */

(function ($, Drupal, drupalSettings) {
  'use strict';

  // Update the online status icon based on connectivity.
  window.addEventListener('online', function () {
    checkStatus();
  });
  window.addEventListener('offline', function () {
    checkStatus();
  });

  /**
   * Check internet status and disable chatbox.
   */
  function checkStatus() {
    var el = $('#chat');

    // If online.
    if (navigator.onLine) {
      toastr.success('All good, internet is back on!');
      /*el.prop('disabled', false);
      el.removeClass('offline').addClass('online');*/
      triggerRefresh();
    }
    // If not online.
    else {
      toastr.warning('Internet connection lost.');
      disableFileUpload();
      el.prop('disabled', true);
      el.removeClass('online').addClass('offline');
    }
  }

  /**
   * Trigger a refresh.
   */
  function triggerRefresh() {
    location.reload();
  }

  /**
   * Disable the ability to upload files.
   */
  function disableFileUpload() {
    Dropzone.options.uploadWidget = {
      autoQueue: false,
      autoProcessQueue: false,
      init: function () {
        var wrapperThis = this;
        this.on("addedfile", function (file) {
          wrapperThis.removeFile(file);
          toastr.error('File could not be uploaded.');
        });
      }
    };
  }
}(jQuery, Drupal, drupalSettings));
